require "json"
require "httpclient"
require "open-uri"
require "zlib"
require "rubygems/package"
require "tempfile"
require "yaml"

def self.get_url(url)
	timeout = 1200
	c = HTTPClient.new
	c.ssl_config.verify_mode = OpenSSL::SSL::VERIFY_NONE
	c.ssl_config.timeout = timeout
	connection_exceptions = [HTTPClient::BadResponseError, HTTPClient::ConnectTimeoutError]

	begin
		ssl_tries ||= timeout
		dns_tries ||= timeout
		http_tries ||= timeout
		content = c.get_content(url)
	rescue OpenSSL::SSL::SSLError
		if (ssl_tries -= 1) > 0
			sleep 1
			retry
		else
			puts "ERROR: ELB is up, but instances are not (SSL)"
			$build_failure = true
		end
	rescue *connection_exceptions
		if (http_tries -= 1) > 0
			sleep 1
			retry
		else
			puts "ERROR: ELB is up, but instances are not (HTTP)"
			$build_failure = true
		end
	rescue SocketError
		if (dns_tries -= 1) > 0
			sleep 1
			retry
		else
			puts "ERROR: DNS record not created"
			$build_failure = true
		end
	else
		return content
	end
end

def self.load_production_assets(version, github_username, github_password, tenant_name, domain, socotra_username, socotra_password, api_url, apidoc_url)
	url = "https://github.com/socotra/prime/archive/#{version}.tar.gz"
	assets_dir = Dir.mktmpdir
	assets_artifact = "#{assets_dir}/#{version}.tar.gz"

	open(assets_artifact, "w").write(open(url, :http_basic_authentication => [github_username, github_password]).read)

	tar_extract = Gem::Package::TarReader.new(Zlib::GzipReader.open(assets_artifact))
	tar_extract.rewind
	tar_extract.each do |entry|
		if entry.file?
			FileUtils.mkdir_p(File.dirname("#{assets_dir}/#{entry.full_name}"))
			File.open("#{assets_dir}/#{entry.full_name}", "wb") do |f|
				f.write(entry.read)
			end
			File.chmod(entry.header.mode, "#{assets_dir}/#{entry.full_name}")
		end
	end
	tar_extract.close

	cmd = "socotraadmin tenant add_tenant #{tenant_name} --tenant_hostname=#{tenant_name}.co.#{domain} --tenant_path=#{assets_dir}/prime-#{version} --overwrite_ontology --admin_username=#{socotra_username} --admin_password=#{socotra_password} --apidoc_url=#{apidoc_url} --api_url=#{api_url}"
	system_safe(cmd, "load_prod_assets_for_pr", "Failed to load prod asset for PR")
end

def self.load_tabular_datasources(jwtsecret, api_url, apidoc_url, admin_username, admin_password, tenant_name)
	cmd = "socotraadmin tabular_datasource create_bulk \
              --jwtsecret=#{$jwtsecret} \
              --api_url=#{$api_url} \
              --apidoc_url=#{$apidoc_url} \
              --admin_username=#{$admin_username} \
              --admin_password=#{$admin_password} \
		      #{$tenant_name} \
              $(pwd)/data/tables.json"
    system_safe(cmd, "load_tabular_datasources", "Failed to load tabular datasources")
end

def self.load_assets_from_branch(tenant_name, domain, admin_username, admin_password, api_url, apidoc_url)
	if api_url == "https://api.staging.socotra.com"
		cmd = "socotraadmin tenant add_tenant #{tenant_name} --overwrite_ontology --tenant_hostname=#{tenant_name}.co.#{domain} --tenant_path=. --admin_username=#{admin_username} --admin_password=#{admin_password} --apidoc_url=#{apidoc_url} --api_url=#{api_url}"
	else
		cmd = "socotraadmin tenant add_tenant #{tenant_name} --tenant_hostname=#{tenant_name}.co.#{domain} --tenant_path=. --admin_username=#{admin_username} --admin_password=#{admin_password} --apidoc_url=#{apidoc_url} --api_url=#{api_url}"
	end
		
	system_safe(cmd, "load_branch_assets_for_pr", "Failed to load prod asset for PR")
end

def self.load_tasks(version, environment, tenant_name)
	cmd = "sg docker -c \'tasker --path=reports --tag=#{$version} --environment=#{$environment} --timezone='Africa/Kigali' --tenant=#{$tenant_name} -v\'"
    system_safe(cmd, "load_tasks", "Failed to load task for PR")
end


